@@ -1,23 +1,21 @@
 """Pytorch based computation backend."""
 
+import math
 from functools import wraps
 
 import numpy as _np
 import torch
-from torch import (  # NOQA
-    acos as arccos,
-    arange,
-    argmin,
-    asin as arcsin,
-    atan2 as arctan2,
-    bool as t_bool,
-    broadcast_tensors as broadcast_arrays,
+from torch import arange, arccos, arccosh, arcsin, arctanh, argmin
+from torch import atan2 as arctan2  # NOQA
+from torch import bool as t_bool
+from torch import broadcast_tensors as broadcast_arrays
+from torch import (
     ceil,
-    clamp as clip,
+    clip,
     cos,
     cosh,
     cross,
-    div as divide,
+    divide,
     empty_like,
     eq,
     erf,
@@ -27,49 +25,54 @@ from torch import (  # NOQA
     float32,
     float64,
     floor,
-    fmod as mod,
-    ger as outer,
-    gt as greater,
+)
+from torch import fmod as mod
+from torch import (
+    greater,
+    hstack,
+    imag,
     int32,
     int64,
     isnan,
+    less,
     log,
-    lt as less,
+    logical_or,
     matmul,
-    max as amax,
-    mean,
-    meshgrid,
-    min as amin,
-    nonzero,
-    ones,
-    ones_like,
-    polygamma,
-    pow as power,
-    repeat_interleave as repeat,
+)
+from torch import max as amax
+from torch import mean, meshgrid
+from torch import min as amin
+from torch import nonzero, ones, ones_like, outer, polygamma
+from torch import pow as power
+from torch import real
+from torch import repeat_interleave as repeat
+from torch import (
     reshape,
     sign,
     sin,
     sinh,
+    sort,
     stack,
     std,
     tan,
     tanh,
     tril,
     uint8,
+    vstack,
     zeros,
-    zeros_like
+    zeros_like,
 )
 
-from . import autograd # NOQA
+from ..constants import pytorch_atol, pytorch_rtol
+from . import autodiff  # NOQA
 from . import linalg  # NOQA
 from . import random  # NOQA
 
+DTYPES = {int32: 0, int64: 1, float32: 2, float64: 3}
 
-DTYPES = {
-    int32: 0,
-    int64: 1,
-    float32: 2,
-    float64: 3}
+
+atol = pytorch_atol
+rtol = pytorch_rtol
 
 
 def _raise_not_implemented_error(*args, **kwargs):
@@ -85,6 +88,7 @@ def _box_scalar(function):
         if not torch.is_tensor(x):
             x = torch.tensor(x)
         return function(x)
+
     return wrapper
 
 
@@ -93,16 +97,32 @@ ceil = _box_scalar(ceil)
 cos = _box_scalar(cos)
 cosh = _box_scalar(cosh)
 exp = _box_scalar(exp)
+imag = _box_scalar(imag)
 log = _box_scalar(log)
+real = _box_scalar(real)
 sin = _box_scalar(sin)
 sinh = _box_scalar(sinh)
 tan = _box_scalar(tan)
 
 
+def comb(n, k):
+    return math.factorial(n) // math.factorial(k) // math.factorial(n - k)
+
+
 def to_numpy(x):
     return x.numpy()
 
 
+def from_numpy(x):
+    return torch.from_numpy(x)
+
+
+def one_hot(labels, num_classes):
+    if not torch.is_tensor(labels):
+        labels = torch.LongTensor(labels)
+    return torch.nn.functional.one_hot(labels, num_classes).type(torch.uint8)
+
+
 def argmax(a, **kwargs):
     if a.dtype == torch.bool:
         return torch.as_tensor(_np.argmax(a.data.numpy(), **kwargs))
@@ -144,13 +164,9 @@ def split(x, indices_or_sections, axis=0):
     return torch.split(x, tuple(intervals_length), dim=axis)
 
 
-def logical_or(x, y):
-    return x or y
-
-
 def logical_and(x, y):
     if torch.is_tensor(x):
-        return x & y
+        return torch.logical_and(x, y)
     return x and y
 
 
@@ -190,21 +206,8 @@ def concatenate(seq, axis=0, out=None):
     return torch.cat(seq, dim=axis, out=out)
 
 
-def hstack(seq):
-    return concatenate(seq, axis=1)
-
-
-def vstack(seq):
-    return concatenate(seq)
-
-
 def _get_largest_dtype(seq):
-    dtype_dict = {0: t_bool,
-                  1: uint8,
-                  2: int32,
-                  3: int64,
-                  4: float32,
-                  5: float64}
+    dtype_dict = {0: t_bool, 1: uint8, 2: int32, 3: int64, 4: float32, 5: float64}
     reverse_dict = {dtype_dict[key]: key for key in dtype_dict}
     dtype_code_set = {reverse_dict[t.dtype] for t in seq}
     return dtype_dict[max(dtype_code_set)]
@@ -296,7 +299,7 @@ def get_slice(x, indices):
     return x[indices]
 
 
-def allclose(a, b, **kwargs):
+def allclose(a, b, atol=atol, rtol=rtol):
     if not isinstance(a, torch.Tensor):
         a = torch.tensor(a)
     if not isinstance(b, torch.Tensor):
@@ -312,21 +315,7 @@ def allclose(a, b, **kwargs):
     elif n_a < n_b:
         reps = (int(n_b / n_a),) + (nb_dim - 1) * (1,)
         a = tile(a, reps)
-    return torch.allclose(a, b, **kwargs)
-
-
-def arccosh(x):
-    c0 = torch.log(x)
-    c1 = torch.log1p(torch.sqrt(x * x - 1) / x)
-    return c0 + c1
-
-
-def arcsinh(x):
-    return torch.log(x + torch.sqrt(x * x + 1))
-
-
-def arcosh(x):
-    return torch.log(x + torch.sqrt(x * x - 1))
+    return torch.allclose(a, b, atol=atol, rtol=rtol)
 
 
 def shape(val):
@@ -334,7 +323,7 @@ def shape(val):
 
 
 def dot(a, b):
-    return einsum('...i,...i->...', a, b)
+    return einsum("...i,...i->...", a, b)
 
 
 def maximum(a, b):
@@ -348,13 +337,19 @@ def to_ndarray(x, to_ndim, axis=0):
     return x
 
 
+def broadcast_to(x, shape):
+    if not torch.is_tensor(x):
+        x = torch.tensor(x)
+    return x.expand(shape)
+
+
 def sqrt(x):
     if not isinstance(x, torch.Tensor):
         x = torch.tensor(x).float()
     return torch.sqrt(x)
 
 
-def isclose(x, y, rtol=1e-5, atol=1e-8):
+def isclose(x, y, rtol=rtol, atol=atol):
     if not torch.is_tensor(x):
         x = torch.tensor(x)
     if not torch.is_tensor(y):
@@ -381,14 +376,14 @@ def einsum(*args, **kwargs):
     if len(input_tensors_list) == 1:
         return torch.einsum(einsum_str, input_tensors_list)
 
-    einsum_list = einsum_str.split('->')
+    einsum_list = einsum_str.split("->")
     input_str = einsum_list[0]
     if len(einsum_list) > 1:
         output_str = einsum_list[1]
 
-    input_str_list = input_str.split(',')
+    input_str_list = input_str.split(",")
 
-    is_ellipsis = [input_str[:3] == '...' for input_str in input_str_list]
+    is_ellipsis = [input_str[:3] == "..." for input_str in input_str_list]
     all_ellipsis = bool(_np.prod(is_ellipsis))
 
     if all_ellipsis:
@@ -396,7 +391,8 @@ def einsum(*args, **kwargs):
 
         if len(input_str_list) > 2:
             raise NotImplementedError(
-                'Ellipsis support not implemented for >2 input tensors')
+                "Ellipsis support not implemented for >2 input tensors"
+            )
 
         tensor_a = input_tensors_list[0]
         tensor_b = input_tensors_list[1]
@@ -411,38 +407,40 @@ def einsum(*args, **kwargs):
         cond = (
             n_tensor_a == n_tensor_b == 1
             and initial_ndim_a != tensor_a.ndim
-            and initial_ndim_b != tensor_b.ndim)
+            and initial_ndim_b != tensor_b.ndim
+        )
 
         if cond:
             tensor_a = squeeze(tensor_a, axis=0)
             tensor_b = squeeze(tensor_b, axis=0)
-            input_prefix_list = ['', '']
-            output_prefix = ''
+            input_prefix_list = ["", ""]
+            output_prefix = ""
         elif n_tensor_a != n_tensor_b:
             if n_tensor_a == 1:
                 tensor_a = squeeze(tensor_a, axis=0)
-                input_prefix_list = ['', 'r']
-                output_prefix = 'r'
+                input_prefix_list = ["", "r"]
+                output_prefix = "r"
             elif n_tensor_b == 1:
                 tensor_b = squeeze(tensor_b, axis=0)
-                input_prefix_list = ['r', '']
-                output_prefix = 'r'
+                input_prefix_list = ["r", ""]
+                output_prefix = "r"
             else:
-                raise ValueError('Shape mismatch for einsum.')
+                raise ValueError("Shape mismatch for einsum.")
         else:
-            input_prefix_list = ['r', 'r']
-            output_prefix = 'r'
+            input_prefix_list = ["r", "r"]
+            output_prefix = "r"
 
         input_str_list = [
-            input_str.replace('...', prefix) for input_str, prefix in zip(
-                input_str_list, input_prefix_list)]
+            input_str.replace("...", prefix)
+            for input_str, prefix in zip(input_str_list, input_prefix_list)
+        ]
 
-        input_str = input_str_list[0] + ',' + input_str_list[1]
+        input_str = input_str_list[0] + "," + input_str_list[1]
 
         einsum_str = input_str
         if len(einsum_list) > 1:
-            output_str = output_str.replace('...', output_prefix)
-            einsum_str = input_str + '->' + output_str
+            output_str = output_str.replace("...", output_prefix)
+            einsum_str = input_str + "->" + output_str
 
         result = torch.einsum(einsum_str, tensor_a, tensor_b, **kwargs)
 
@@ -460,6 +458,8 @@ def transpose(x, axes=None):
         return x.permute(axes)
     if x.dim() == 1:
         return x
+    if x.dim() > 2 and axes is None:
+        return x.permute(tuple(range(x.ndim)[::-1]))
     return x.t()
 
 
@@ -473,20 +473,16 @@ def trace(x, axis1=0, axis2=1):
     min_axis = min(axis1, axis2)
     max_axis = max(axis1, axis2)
     if min_axis == 1 and max_axis == 2:
-        return torch.einsum('...ii', x)
+        return torch.einsum("...ii", x)
     if min_axis == -2 and max_axis == -1:
-        return torch.einsum('...ii', x)
+        return torch.einsum("...ii", x)
     if min_axis == 0 and max_axis == 1:
-        return torch.einsum('ii...', x)
+        return torch.einsum("ii...", x)
     if min_axis == 0 and max_axis == 2:
-        return torch.einsum('i...i', x)
+        return torch.einsum("i...i", x)
     raise NotImplementedError()
 
 
-def arctanh(x):
-    return 0.5 * torch.log((1 + x) / (1 - x))
-
-
 def linspace(start, stop, num):
     return torch.linspace(start=start, end=stop, steps=num)
 
@@ -499,12 +495,20 @@ def equal(a, b, **kwargs):
     return torch.eq(a, b, **kwargs)
 
 
-def tril_indices(*args, **kwargs):
-    return tuple(map(torch.from_numpy, _np.tril_indices(*args, **kwargs)))
+def diag_indices(*args, **kwargs):
+    return tuple(map(torch.from_numpy, _np.diag_indices(*args, **kwargs)))
 
 
-def triu_indices(*args, **kwargs):
-    return tuple(map(torch.from_numpy, _np.triu_indices(*args, **kwargs)))
+def tril_indices(n, k=0, m=None):
+    if m is None:
+        m = n
+    return torch.tril_indices(row=n, col=m, offset=k)
+
+
+def triu_indices(n, k=0, m=None):
+    if m is None:
+        m = n
+    return torch.triu_indices(row=n, col=m, offset=k)
 
 
 def tile(x, y):
@@ -552,15 +556,14 @@ def set_diag(x, new_diag):
     """
     arr_shape = x.shape
     off_diag = (1 - torch.eye(arr_shape[-1])) * x
-    diag = torch.einsum(
-        'ij,...i->...ij', torch.eye(new_diag.shape[-1]), new_diag)
+    diag = torch.einsum("ij,...i->...ij", torch.eye(new_diag.shape[-1]), new_diag)
     return diag + off_diag
 
 
 def prod(x, axis=None):
     if axis is None:
-        return torch.prod(x)
-    return torch.prod(x, dim=axis)
+        axis = 0
+    return torch.prod(x, axis)
 
 
 def where(condition, x=None, y=None):
@@ -643,7 +646,7 @@ def assignment(x, values, indices, axis=0):
     """
     x_new = copy(x)
 
-    use_vectorization = hasattr(indices, '__len__') and len(indices) < ndim(x)
+    use_vectorization = hasattr(indices, "__len__") and len(indices) < ndim(x)
     if _is_boolean(indices):
         x_new[indices] = values
         return x_new
@@ -656,11 +659,10 @@ def assignment(x, values, indices, axis=0):
             len_indices = len(indices) if _is_iterable(indices) else 1
         len_values = len(values) if _is_iterable(values) else 1
         if len_values > 1 and len_values != len_indices:
-            raise ValueError('Either one value or as many values as indices')
+            raise ValueError("Either one value or as many values as indices")
         x_new[indices] = values
     else:
-        indices = tuple(
-            list(indices[:axis]) + [slice(None)] + list(indices[axis:]))
+        indices = tuple(list(indices[:axis]) + [slice(None)] + list(indices[axis:]))
         x_new[indices] = values
     return x_new
 
@@ -694,7 +696,7 @@ def assignment_by_sum(x, values, indices, axis=0):
     """
     x_new = copy(x)
     values = array(values)
-    use_vectorization = hasattr(indices, '__len__') and len(indices) < ndim(x)
+    use_vectorization = hasattr(indices, "__len__") and len(indices) < ndim(x)
     if _is_boolean(indices):
         x_new[indices] += values
         return x_new
@@ -705,11 +707,10 @@ def assignment_by_sum(x, values, indices, axis=0):
         len_indices = len(indices) if _is_iterable(indices) else 1
         len_values = len(values) if _is_iterable(values) else 1
         if len_values > 1 and len_values != len_indices:
-            raise ValueError('Either one value or as many values as indices')
+            raise ValueError("Either one value or as many values as indices")
         x_new[indices] += values
     else:
-        indices = tuple(
-            list(indices[:axis]) + [slice(None)] + list(indices[axis:]))
+        indices = tuple(list(indices[:axis]) + [slice(None)] + list(indices[axis:]))
         x_new[indices] += values
     return x_new
 
@@ -728,8 +729,8 @@ def cumsum(x, axis=None):
 
 def cumprod(x, axis=None):
     if axis is None:
-        return x.flatten().cumprod(dim=0)
-    return torch.cumprod(x, dim=axis)
+        axis = 0
+    return torch.cumprod(x, axis)
 
 
 def array_from_sparse(indices, data, target_shape):
@@ -754,7 +755,8 @@ def array_from_sparse(indices, data, target_shape):
     return torch.sparse.FloatTensor(
         torch.LongTensor(indices).t(),
         torch.FloatTensor(cast(data, float32)),
-        torch.Size(target_shape)).to_dense()
+        torch.Size(target_shape),
+    ).to_dense()
 
 
 def vectorize(x, pyfunc, multiple_args=False, **kwargs):
@@ -767,3 +769,29 @@ def triu_to_vec(x, k=0):
     n = x.shape[-1]
     rows, cols = triu_indices(n, k=k)
     return x[..., rows, cols]
+
+
+def mat_from_diag_triu_tril(diag, tri_upp, tri_low):
+    """Build matrix from given components.
+
+    Forms a matrix from diagonal, strictly upper triangular and
+    strictly lower traingular parts.
+
+    Parameters
+    ----------
+    diag : array_like, shape=[..., n]
+    tri_upp : array_like, shape=[..., (n * (n - 1)) / 2]
+    tri_low : array_like, shape=[..., (n * (n - 1)) / 2]
+
+    Returns
+    -------
+    mat : array_like, shape=[..., n, n]
+    """
+    n = diag.shape[-1]
+    (i,) = diag_indices(n, ndim=1)
+    j, k = triu_indices(n, k=1)
+    mat = torch.zeros((diag.shape + (n,)))
+    mat[..., i, i] = diag
+    mat[..., j, k] = tri_upp
+    mat[..., k, j] = tri_low
+    return mat
