@@ -8,9 +8,7 @@ of the matrices (and a -1 in its lower triangular part), except in dim 2 and
 
 import geomstats.backend as gs
 from geomstats.geometry.lie_algebra import MatrixLieAlgebra
-
-
-TOLERANCE = 1e-8
+from geomstats.geometry.matrices import Matrices
 
 
 class SkewSymmetricMatrices(MatrixLieAlgebra):
@@ -25,30 +23,31 @@ class SkewSymmetricMatrices(MatrixLieAlgebra):
     def __init__(self, n):
         dim = int(n * (n - 1) / 2)
         super(SkewSymmetricMatrices, self).__init__(dim, n)
+        self.ambient_space = Matrices(n, n)
 
         if n == 2:
-            self.basis = gs.array([[[0., -1.], [1., 0.]]])
+            self.basis = gs.array([[[0.0, -1.0], [1.0, 0.0]]])
         elif n == 3:
-            self.basis = gs.array([
-                [[0., 0., 0.],
-                 [0., 0., -1.],
-                 [0., 1., 0.]],
-                [[0., 0., 1.],
-                 [0., 0., 0.],
-                 [-1., 0., 0.]],
-                [[0., -1., 0.],
-                 [1., 0., 0.],
-                 [0., 0., 0.]]])
+            self.basis = gs.array(
+                [
+                    [[0.0, 0.0, 0.0], [0.0, 0.0, -1.0], [0.0, 1.0, 0.0]],
+                    [[0.0, 0.0, 1.0], [0.0, 0.0, 0.0], [-1.0, 0.0, 0.0]],
+                    [[0.0, -1.0, 0.0], [1.0, 0.0, 0.0], [0.0, 0.0, 0.0]],
+                ]
+            )
         else:
             self.basis = gs.zeros((dim, n, n))
             basis = []
             for row in gs.arange(n - 1):
                 for col in gs.arange(row + 1, n):
-                    basis.append(gs.array_from_sparse(
-                        [(row, col), (col, row)], [1., -1.], (n, n)))
+                    basis.append(
+                        gs.array_from_sparse(
+                            [(row, col), (col, row)], [1.0, -1.0], (n, n)
+                        )
+                    )
             self.basis = gs.stack(basis)
 
-    def belongs(self, mat, atol=TOLERANCE):
+    def belongs(self, mat, atol=gs.atol):
         """Evaluate if mat is a skew-symmetric matrix.
 
         Parameters
@@ -57,16 +56,38 @@ class SkewSymmetricMatrices(MatrixLieAlgebra):
             Square matrix to check.
         atol : float
             Tolerance for the equality evaluation.
-            Optional, default: TOLERANCE.
+            Optional, default: backend atol.
 
         Returns
         -------
         belongs : array-like, shape=[...,]
             Boolean evaluating if matrix is skew symmetric.
         """
-        is_skew = self.is_skew_symmetric(mat=mat, atol=atol)
-        return gs.logical_and(
-            is_skew, super(SkewSymmetricMatrices, self).belongs(mat))
+        has_right_shape = self.ambient_space.belongs(mat)
+        if gs.all(has_right_shape):
+            return Matrices.is_skew_symmetric(mat=mat, atol=atol)
+        return has_right_shape
+
+    def random_point(self, n_samples=1, bound=1.0):
+        """Sample from a uniform distribution in a cube.
+
+        Parameters
+        ----------
+        n_samples : int
+            Number of samples.
+            Optional, default: 1.
+        bound : float
+            Bound of the interval in which to sample each entry.
+            Optional, default: 1.
+
+        Returns
+        -------
+        point : array-like, shape=[..., n, n]
+            Sample.
+        """
+        return self.projection(
+            super(SkewSymmetricMatrices, self).random_point(n_samples, bound)
+        )
 
     @classmethod
     def projection(cls, mat):
@@ -86,7 +107,7 @@ class SkewSymmetricMatrices(MatrixLieAlgebra):
         skew_sym : array-like, shape=[..., n, n]
             Skew-symmetric matrix.
         """
-        return cls.to_skew_symmetric(mat)
+        return Matrices.to_skew_symmetric(mat)
 
     def basis_representation(self, matrix_representation):
         """Calculate the coefficients of given matrix in the basis.
@@ -107,10 +128,13 @@ class SkewSymmetricMatrices(MatrixLieAlgebra):
         if self.n == 2:
             return matrix_representation[..., 1, 0][..., None]
         if self.n == 3:
-            vec = gs.stack([
-                matrix_representation[..., 2, 1],
-                matrix_representation[..., 0, 2],
-                matrix_representation[..., 1, 0]])
+            vec = gs.stack(
+                [
+                    matrix_representation[..., 2, 1],
+                    matrix_representation[..., 0, 2],
+                    matrix_representation[..., 1, 0],
+                ]
+            )
             return gs.transpose(vec)
 
         return gs.triu_to_vec(matrix_representation, k=1)
