@@ -8,18 +8,11 @@ from sklearn.base import BaseEstimator
 import geomstats.backend as gs
 import geomstats.errors as error
 import geomstats.vectorization
-from geomstats.geometry.euclidean import EuclideanMetric
-from geomstats.geometry.matrices import MatricesMetric
-from geomstats.geometry.minkowski import MinkowskiMetric
 
 EPSILON = 1e-4
 
 
-def variance(points,
-             base_point,
-             metric,
-             weights=None,
-             point_type='vector'):
+def variance(points, base_point, metric, weights=None, point_type="vector"):
     """Variance of (weighted) points wrt a base point.
 
     Parameters
@@ -35,8 +28,7 @@ def variance(points,
     var : float
        Weighted variance of the points.
     """
-    n_points = geomstats.vectorization.get_n_points(
-        points, point_type)
+    n_points = geomstats.vectorization.get_n_points(points, point_type)
 
     if weights is None:
         weights = gs.ones((n_points,))
@@ -51,7 +43,7 @@ def variance(points,
     return var
 
 
-def linear_mean(points, weights=None, point_type='vector'):
+def linear_mean(points, weights=None, point_type="vector"):
     """Compute the weighted linear mean.
 
     The linear mean is the Frechet mean when points:
@@ -74,18 +66,17 @@ def linear_mean(points, weights=None, point_type='vector'):
     if isinstance(points, list):
         points = gs.stack(points, axis=0)
     if isinstance(weights, list):
-        weights = gs.stack(weights, axis=0)
+        weights = gs.array(weights)
 
-    n_points = geomstats.vectorization.get_n_points(
-        points, point_type)
+    n_points = geomstats.vectorization.get_n_points(points, point_type)
 
     if weights is None:
         weights = gs.ones((n_points,))
     sum_weights = gs.sum(weights)
 
-    einsum_str = '...,...j->...j'
-    if point_type == 'matrix':
-        einsum_str = '...,...jk->...jk'
+    einsum_str = "...,...j->...j"
+    if point_type == "matrix":
+        einsum_str = "...,...jk->...jk"
 
     weighted_points = gs.einsum(einsum_str, weights, points)
 
@@ -93,15 +84,16 @@ def linear_mean(points, weights=None, point_type='vector'):
     return mean
 
 
-def _default_gradient_descent(points, metric, weights,
-                              max_iter, point_type, epsilon, verbose):
+def _default_gradient_descent(
+    points, metric, weights, max_iter, point_type, epsilon, initial_step_size, verbose
+):
     """Perform default gradient descent."""
-    if point_type == 'vector':
+    if point_type == "vector":
         points = gs.to_ndarray(points, to_ndim=2)
-        einsum_str = 'n,nj->j'
+        einsum_str = "n,nj->j"
     else:
         points = gs.to_ndarray(points, to_ndim=3)
-        einsum_str = 'n,nij->ij'
+        einsum_str = "n,nij->ij"
     n_points = gs.shape(points)[0]
 
     if weights is None:
@@ -115,129 +107,144 @@ def _default_gradient_descent(points, metric, weights,
     sum_weights = gs.sum(weights)
     sq_dists_between_iterates = []
     iteration = 0
-    sq_dist = 0.
-    var = 0.
+    sq_dist = 0.0
+    var = 0.0
+
+    norm_old = gs.linalg.norm(points)
+    step = initial_step_size
 
     while iteration < max_iter:
-        var_is_0 = gs.isclose(var, 0.)
-        sq_dist_is_small = gs.less_equal(sq_dist, epsilon * var)
-        condition = ~gs.logical_or(var_is_0, sq_dist_is_small)
-        if not (condition or iteration == 0):
-            break
-
         logs = metric.log(point=points, base_point=mean)
 
+        var = gs.sum(metric.squared_norm(logs, mean) * weights) / gs.sum(weights)
+
         tangent_mean = gs.einsum(einsum_str, weights, logs)
         tangent_mean /= sum_weights
-
-        estimate_next = metric.exp(tangent_vec=tangent_mean, base_point=mean)
+        norm = gs.linalg.norm(tangent_mean)
 
         sq_dist = metric.squared_norm(tangent_mean, mean)
         sq_dists_between_iterates.append(sq_dist)
 
-        var = variance(
-            points=points,
-            weights=weights,
-            metric=metric,
-            base_point=estimate_next,
-            point_type=point_type)
+        var_is_0 = gs.isclose(var, 0.0)
+        sq_dist_is_small = gs.less_equal(sq_dist, epsilon * metric.dim)
+        condition = ~gs.logical_or(var_is_0, sq_dist_is_small)
+        if not (condition or iteration == 0):
+            break
 
+        estimate_next = metric.exp(step * tangent_mean, mean)
         mean = estimate_next
         iteration += 1
 
+        if norm < norm_old:
+            norm_old = norm
+        elif norm > norm_old:
+            step = step / 2.0
+
     if iteration == max_iter:
         logging.warning(
-            'Maximum number of iterations {} reached. '
-            'The mean may be inaccurate'.format(max_iter))
+            "Maximum number of iterations {} reached. "
+            "The mean may be inaccurate".format(max_iter)
+        )
 
     if verbose:
-        logging.info('n_iter: {}, final variance: {}, final dist: {}'.format(
-            iteration, var, sq_dist))
+        logging.info(
+            "n_iter: {}, final variance: {}, final dist: {}".format(
+                iteration, var, sq_dist
+            )
+        )
 
     return mean
 
 
-def _ball_gradient_descent(points, metric, weights=None, max_iter=32,
-                           lr=1e-3, tau=5e-3):
-    """Perform ball gradient descent."""
-    if len(points) == 1:
-        return points
-    points = gs.to_ndarray(points, to_ndim=2)
-    if weights is None:
-
-        iteration = 0
-        convergence = math.inf
-        barycenter = gs.mean(points, axis=0, keepdims=True)
-
-        while convergence > tau and max_iter > iteration:
-
-            iteration += 1
-            grad_tangent = 2 * metric.log(points, barycenter)
-            cc_barycenter = metric.exp(
-                lr * grad_tangent.sum(0, keepdims=True), barycenter)
-
-            convergence = metric.dist(cc_barycenter, barycenter).max().item()
-
-            barycenter = cc_barycenter
+def _batch_gradient_descent(
+    points,
+    metric,
+    weights=None,
+    max_iter=32,
+    lr=1e-3,
+    epsilon=5e-3,
+    point_type="vector",
+    verbose=False,
+):
+    """Perform batch gradient descent."""
+    if point_type == "vector":
+        if points.ndim < 3:
+            return _default_gradient_descent(
+                points, metric, weights, max_iter, point_type, epsilon, lr, verbose
+            )
+        einsum_str = "ni,nij->ij"
+        ndim = 1
     else:
+        if points.ndim < 4:
+            return _default_gradient_descent(
+                points, metric, weights, max_iter, point_type, epsilon, lr, verbose
+            )
+        einsum_str = "nk,nkij->kij"
+        ndim = 2
 
-        weights = gs.expand_dims(weights, -1)
-        weights = gs.repeat(weights, points.shape[-1], axis=2)
+    shape = points.shape
+    n_points = shape[0]
+    n_batch = shape[1]
 
-        barycenter = (points * weights).sum(0, keepdims=True) / weights.sum(0)
-        barycenter_gs = gs.squeeze(barycenter)
+    if n_points == 1:
+        return points[0]
 
-        points_gs = gs.squeeze(points)
-        points_flattened = gs.reshape(points_gs, (-1, points_gs.shape[-1]))
+    if weights is None:
+        weights = gs.ones((n_points, n_batch))
 
-        convergence = math.inf
-        iteration = 0
+    flat_shape = (n_batch * n_points,) + shape[-ndim:]
+    estimates = points[0]
+    points_flattened = gs.reshape(points, (n_points * n_batch,) + shape[-ndim:])
+    convergence = math.inf
+    iteration = 0
+    convergence_old = convergence
 
-        while convergence > tau and max_iter > iteration:
+    while convergence > epsilon and max_iter > iteration:
 
-            iteration += 1
+        iteration += 1
+        estimates_broadcast, _ = gs.broadcast_arrays(estimates, points)
+        estimates_flattened = gs.reshape(estimates_broadcast, flat_shape)
 
-            barycenter_flattened = gs.repeat(barycenter,
-                                             len(points_gs), axis=0)
-            barycenter_flattened = gs.reshape(
-                barycenter_flattened,
-                (-1, barycenter_flattened.shape[-1]))
+        tangent_grad = metric.log(points_flattened, estimates_flattened)
+        tangent_grad = gs.reshape(tangent_grad, shape)
 
-            grad_tangent = 2 * metric.log(points_flattened,
-                                          barycenter_flattened)
-            grad_tangent = gs.reshape(grad_tangent,
-                                      points.shape)
-            grad_tangent = grad_tangent * weights
+        tangent_mean = gs.einsum(einsum_str, weights, tangent_grad) / n_points
 
-            lr_grad_tangent = lr * grad_tangent.sum(0, keepdims=True)
-            lr_grad_tangent_s = lr_grad_tangent.squeeze()
+        next_estimates = metric.exp(lr * tangent_mean, estimates)
+        convergence = gs.sum(metric.squared_norm(tangent_mean, estimates))
+        estimates = next_estimates
 
-            cc_barycenter = metric.exp(barycenter_gs,
-                                       lr_grad_tangent_s)
-
-            convergence = metric.dist(cc_barycenter,
-                                      barycenter_gs).max().item()
-
-            barycenter_gs = cc_barycenter
-            barycenter = gs.expand_dims(cc_barycenter, 0)
-
-        barycenter = gs.squeeze(barycenter)
+        if convergence < convergence_old:
+            convergence_old = convergence
+        elif convergence > convergence_old:
+            lr = lr / 2.0
 
     if iteration == max_iter:
         logging.warning(
-            'Maximum number of iterations {} reached. The '
-            'mean may be inaccurate'.format(max_iter))
+            "Maximum number of iterations {} reached. The "
+            "mean may be inaccurate".format(max_iter)
+        )
 
-    return barycenter
+    if verbose:
+        logging.info(
+            "n_iter: {}, final dist: {},"
+            "final step size: {}".format(iteration, convergence, lr)
+        )
+
+    return estimates
 
 
-def _adaptive_gradient_descent(points,
-                               metric,
-                               weights=None,
-                               max_iter=32,
-                               epsilon=1e-12,
-                               init_point=None,
-                               point_type='vector'):
+def _adaptive_gradient_descent(
+    points,
+    metric,
+    weights=None,
+    max_iter=32,
+    epsilon=1e-12,
+    initial_tau=1.0,
+    init_point=None,
+    point_type="vector",
+    verbose=False,
+):
     """Perform adaptive gradient descent.
 
     Frechet mean of (weighted) points using adaptive time-steps
@@ -265,52 +272,56 @@ def _adaptive_gradient_descent(points,
     current_mean: array-like, shape=[..., dim]
         Weighted Frechet mean of the points.
     """
-    if point_type == 'matrix':
-        raise NotImplementedError(
-            'The Frechet mean with adaptive gradient descent is only'
-            ' implemented for lists of vectors, and not matrices.')
+    if point_type == "vector":
+        points = gs.to_ndarray(points, to_ndim=2)
+        einsum_str = "n,nj->j"
+    else:
+        points = gs.to_ndarray(points, to_ndim=3)
+        einsum_str = "n,nij->ij"
+    n_points = gs.shape(points)[0]
 
     tau_max = 1e6
     tau_mul_up = 1.6511111
     tau_min = 1e-6
     tau_mul_down = 0.1
 
-    n_points = geomstats.vectorization.get_n_points(
-        points, point_type)
-
-    points = gs.to_ndarray(points, to_ndim=2)
+    if n_points == 1:
+        return points[0]
 
     current_mean = points[0] if init_point is None else init_point
 
-    if n_points == 1:
-        return current_mean
-
     if weights is None:
         weights = gs.ones((n_points,))
     sum_weights = gs.sum(weights)
 
-    tau = 1.0
+    tau = initial_tau
     iteration = 0
 
     logs = metric.log(point=points, base_point=current_mean)
-    current_tangent_mean = gs.einsum('n,nj->j', weights, logs)
+    var = gs.sum(metric.squared_norm(logs, current_mean) * weights) / gs.sum(weights)
+
+    current_tangent_mean = gs.einsum(einsum_str, weights, logs)
     current_tangent_mean /= sum_weights
     sq_norm_current_tangent_mean = metric.squared_norm(
-        current_tangent_mean, base_point=current_mean)
+        current_tangent_mean, base_point=current_mean
+    )
 
-    while (sq_norm_current_tangent_mean > epsilon ** 2
-           and iteration < max_iter):
+    while sq_norm_current_tangent_mean > epsilon ** 2 and iteration < max_iter:
         iteration += 1
 
         shooting_vector = tau * current_tangent_mean
-        next_mean = metric.exp(
-            tangent_vec=shooting_vector, base_point=current_mean)
+        next_mean = metric.exp(tangent_vec=shooting_vector, base_point=current_mean)
 
         logs = metric.log(point=points, base_point=next_mean)
-        next_tangent_mean = gs.einsum('n,nj->j', weights, logs)
+        var = gs.sum(metric.squared_norm(logs, current_mean) * weights) / gs.sum(
+            weights
+        )
+
+        next_tangent_mean = gs.einsum(einsum_str, weights, logs)
         next_tangent_mean /= sum_weights
         sq_norm_next_tangent_mean = metric.squared_norm(
-            next_tangent_mean, base_point=next_mean)
+            next_tangent_mean, base_point=next_mean
+        )
 
         if sq_norm_next_tangent_mean < sq_norm_current_tangent_mean:
             current_mean = next_mean
@@ -322,8 +333,18 @@ def _adaptive_gradient_descent(points,
 
     if iteration == max_iter:
         logging.warning(
-            'Maximum number of iterations {} reached. '
-            'The mean may be inaccurate'.format(max_iter))
+            "Maximum number of iterations {} reached. "
+            "The mean may be inaccurate".format(max_iter)
+        )
+
+    if verbose:
+        logging.info(
+            "n_iter: {}, final variance: {}, final dist: {},"
+            " final_step_size: {}".format(
+                iteration, var, sq_norm_current_tangent_mean, tau
+            )
+        )
+
     return current_mean
 
 
@@ -340,22 +361,28 @@ class FrechetMean(BaseEstimator):
     point_type : str, {\'vector\', \'matrix\'}
         Point type.
         Optional, default: None.
-    method : str, {\'default\', \'adaptive\', \'ball\'}
+    method : str, {\'default\', \'adaptive\', \'batch\'}
         Gradient descent method.
+        The `adaptive` method uses a Levenberg-Marquardt style adaptation of
+        the learning rate. The `batch` method is similar to the default
+        method but for batches of equal length of samples. In this case,
+        samples must be of shape [n_samples, n_batch, {dim, [n,n]}].
         Optional, default: \'default\'.
     verbose : bool
         Verbose option.
         Optional, default: False.
     """
 
-    def __init__(self, metric,
-                 max_iter=32,
-                 epsilon=EPSILON,
-                 point_type=None,
-                 method='default',
-                 lr=1e-3,
-                 tau=5e-3,
-                 verbose=False):
+    def __init__(
+        self,
+        metric,
+        max_iter=32,
+        epsilon=EPSILON,
+        point_type=None,
+        method="default",
+        lr=1.0,
+        verbose=False,
+    ):
 
         self.metric = metric
         self.max_iter = max_iter
@@ -363,21 +390,21 @@ class FrechetMean(BaseEstimator):
         self.point_type = point_type
         self.method = method
         self.lr = lr
-        self.tau = tau
         self.verbose = verbose
         self.estimate_ = None
 
         if point_type is None:
             self.point_type = metric.default_point_type
         error.check_parameter_accepted_values(
-            self.point_type, 'point_type', ['vector', 'matrix'])
+            self.point_type, "point_type", ["vector", "matrix"]
+        )
 
     def fit(self, X, y=None, weights=None):
         """Compute the empirical Frechet mean.
 
         Parameters
         ----------
-        X : {array-like, sparse matrix}, shape=[..., n_features]
+        X : {array-like, sparse matrix}, shape=[..., {dim, [n, n]}]
             Training input samples.
         y : array-like, shape=[...,] or [..., n_outputs]
             Target values (class labels in classification, real numbers in
@@ -392,28 +419,53 @@ class FrechetMean(BaseEstimator):
         self : object
             Returns self.
         """
-        is_linear_metric = isinstance(
-            self.metric, (EuclideanMetric, MatricesMetric, MinkowskiMetric))
+        metric_str = self.metric.__str__()
+        is_linear_metric = (
+            "EuclideanMetric" in metric_str
+            or "MatricesMetric" in metric_str
+            or "MinkowskiMetric" in metric_str
+        )
+
+        error.check_parameter_accepted_values(
+            self.method, "method", ["default", "adaptive", "batch"]
+        )
 
         if is_linear_metric:
-            mean = linear_mean(
-                points=X, weights=weights, point_type=self.point_type)
+            mean = linear_mean(points=X, weights=weights, point_type=self.point_type)
 
-        elif self.method == 'default':
+        elif self.method == "default":
             mean = _default_gradient_descent(
-                points=X, weights=weights, metric=self.metric,
+                points=X,
+                weights=weights,
+                metric=self.metric,
                 max_iter=self.max_iter,
-                point_type=self.point_type, epsilon=self.epsilon,
-                verbose=self.verbose)
-        elif self.method == 'adaptive':
+                initial_step_size=self.lr,
+                point_type=self.point_type,
+                epsilon=self.epsilon,
+                verbose=self.verbose,
+            )
+        elif self.method == "adaptive":
             mean = _adaptive_gradient_descent(
-                points=X, weights=weights, metric=self.metric,
+                points=X,
+                weights=weights,
+                metric=self.metric,
                 max_iter=self.max_iter,
-                epsilon=1e-12)
-        elif self.method == 'frechet-poincare-ball':
-            mean = _ball_gradient_descent(
-                points=X, weights=weights, metric=self.metric,
-                lr=self.lr, tau=self.tau, max_iter=self.max_iter)
+                point_type=self.point_type,
+                epsilon=self.epsilon,
+                verbose=self.verbose,
+                initial_tau=self.lr,
+            )
+        elif self.method == "batch":
+            mean = _batch_gradient_descent(
+                points=X,
+                weights=weights,
+                metric=self.metric,
+                lr=self.lr,
+                epsilon=self.epsilon,
+                max_iter=self.max_iter,
+                point_type=self.point_type,
+                verbose=self.verbose,
+            )
 
         self.estimate_ = mean
 
