@@ -31,6 +31,13 @@ class RiemannianKMeans(TransformerMixin, ClusterMixin, BaseEstimator):
         Convergence factor. Convergence is achieved when the difference of mean
         distance between two steps is lower than tol.
         Optional, default: 1e-2.
+    max_iter : int
+        Maximum number of iterations.
+        Optional, default: 100
+    max_iter_mean : int
+        Maximum number of iterations for the gradient descent of each Frechet
+        mean.
+        Optional, default: 100.
     verbose : int
         If verbose > 0, information will be printed during learning.
         Optional, default: 0.
@@ -42,19 +49,31 @@ class RiemannianKMeans(TransformerMixin, ClusterMixin, BaseEstimator):
     """
 
     def __init__(
-            self, metric, n_clusters=8, init='random',
-            tol=1e-2, mean_method='default', verbose=0, point_type='vector'):
+        self,
+        metric,
+        n_clusters=8,
+        init="random",
+        lr=1.0,
+        tol=1e-2,
+        max_iter=100,
+        max_iter_mean=100,
+        mean_method="default",
+        verbose=0,
+    ):
         self.n_clusters = n_clusters
         self.init = init
         self.metric = metric
         self.tol = tol
+        self.lr = lr
         self.verbose = verbose
         self.mean_method = mean_method
-        self.point_type = point_type
+        self.point_type = metric.default_point_type
+        self.max_iter = max_iter
+        self.max_iter_mean = max_iter_mean
 
         self.centroids = None
 
-    def fit(self, X, max_iter=100):
+    def fit(self, X):
         """Provide clusters centroids and data labels.
 
         Alternate between computing the mean of each cluster
@@ -75,16 +94,19 @@ class RiemannianKMeans(TransformerMixin, ClusterMixin, BaseEstimator):
             Centroids.
         """
         n_samples = X.shape[0]
-        self.centroids = [gs.expand_dims(X[randint(0, n_samples - 1)], 0)
-                          for i in range(self.n_clusters)]
+        self.centroids = [
+            gs.expand_dims(X[randint(0, n_samples - 1)], 0)
+            for i in range(self.n_clusters)
+        ]
         self.centroids = gs.concatenate(self.centroids, axis=0)
         index = 0
-        while index < max_iter:
+        while index < self.max_iter:
             index += 1
 
-            dists = [gs.to_ndarray(
-                     self.metric.dist(self.centroids[i], X), 2, 1)
-                     for i in range(self.n_clusters)]
+            dists = [
+                gs.to_ndarray(self.metric.dist(self.centroids[i], X), 2, 1)
+                for i in range(self.n_clusters)
+            ]
             dists = gs.hstack(dists)
             belongs = gs.argmin(dists, 1)
             old_centroids = gs.copy(self.centroids)
@@ -96,30 +118,34 @@ class RiemannianKMeans(TransformerMixin, ClusterMixin, BaseEstimator):
                     mean = FrechetMean(
                         metric=self.metric,
                         method=self.mean_method,
-                        max_iter=150,
-                        point_type=self.point_type)
+                        max_iter=self.max_iter_mean,
+                        lr=self.lr,
+                        point_type=self.point_type,
+                    )
                     mean.fit(fold)
 
                     self.centroids[i] = mean.estimate_
                 else:
                     self.centroids[i] = X[randint(0, n_samples - 1)]
 
-            centroids_distances = self.metric.dist(
-                old_centroids, self.centroids)
+            centroids_distances = self.metric.dist(old_centroids, self.centroids)
 
             if gs.mean(centroids_distances) < self.tol:
                 if self.verbose > 0:
-                    logging.info('Convergence reached after {} '
-                                 'iterations'.format(index))
+                    logging.info(
+                        "Convergence reached after {} " "iterations".format(index)
+                    )
 
                 if self.n_clusters == 1:
                     self.centroids = gs.squeeze(self.centroids, axis=0)
 
                 return gs.copy(self.centroids)
 
-        if index == max_iter:
-            logging.warning('K-means maximum number of iterations {} reached. '
-                            'The mean may be inaccurate'.format(max_iter))
+        if index == self.max_iter:
+            logging.warning(
+                "K-means maximum number of iterations {} reached. "
+                "The mean may be inaccurate".format(self.max_iter)
+            )
 
         if self.n_clusters == 1:
             self.centroids = gs.squeeze(self.centroids, axis=0)
@@ -142,11 +168,10 @@ class RiemannianKMeans(TransformerMixin, ClusterMixin, BaseEstimator):
             Array of predicted cluster indices for each sample.
         """
         if self.centroids is None:
-            raise RuntimeError('fit needs to be called first.')
+            raise RuntimeError("fit needs to be called first.")
         dists = gs.stack(
-            [self.metric.dist(centroid, X)
-             for centroid in self.centroids],
-            axis=1)
+            [self.metric.dist(centroid, X) for centroid in self.centroids], axis=1
+        )
         dists = gs.squeeze(dists)
 
         belongs = gs.argmin(dists, -1)
